/*
Dual: a simple manager for custom hostnames

Copyright (C) 2021 luca0N!

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.

Contact me by e-mail via <luca0n [at] luca0n [dot] com>.
*/

package com.luca0n.dual.config;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.nio.file.Path;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.io.IOException;

import com.luca0n.dual.common.ConfigSyntaxException;
import com.luca0n.dual.config.Config;

/**
 * @since @2021-03-21
 */
public class ConfigUtil {
	public static final String DEFAULT_CONFIG = "gen.cfg";
	public static Config fromFile(String filename)
			throws IOException, ConfigSyntaxException{
		Path path = FileSystems.getDefault().getPath("cfg", filename);
		byte[] fileBytes = Files.readAllBytes(path);

		String[] lines = new String(fileBytes).split(System.lineSeparator());

		Map<String, String> v = new HashMap<>();
		List<Config.Namespace> n = new ArrayList<>();
		Config.Namespace currentNamespace = null;

		for (String line : lines){
			// Ignore commented lines.
			if (line.startsWith("#"))
				continue;

			// Ignore comments.
			String content = line.split("#")[0];

			// Check if this is a namespace declaration.
			if (content.startsWith("[")){
				// This line must end with a "]".
				if (!content.endsWith("]"))
					throw new ConfigSyntaxException("Invalid namespace declaration: missing \"]\"");
				// Append current namespace to the namespace array if it exists.
				if (currentNamespace != null)
					n.add(currentNamespace);
				// Create a new namespace.
				String nsName = content.substring(1, content.length() - 1);
				currentNamespace = new Config.Namespace(nsName);
			}

			// Check if this is a K/V line.
			if (content.contains("=")){
				String[] contentSplit = content.split("=");
				String key = contentSplit[0],
					value = contentSplit[1];
				currentNamespace.putEntry(key, value);
			}
		}

		// Add the last namespace to the namespace array.
		n.add(currentNamespace);

		Config c = new Config(n);
		return c;
	}
}
